'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var inquirer = _interopDefault(require('inquirer'));
var chalk = _interopDefault(require('chalk'));
var path = _interopDefault(require('path'));
var fs = _interopDefault(require('fs'));
var childProcess = _interopDefault(require('child_process'));
var ncu = _interopDefault(require('npm-check-updates'));

var exec$1 = childProcess.exec;
// ----------------------------------------------------------------
var installModule = (function (module, name, cb) {
  fs.stat(path.resolve('./node_modules/' + module), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      console.log(chalk.yellow('Local ' + name + ' not detected...'));
      inquirer.prompt([{
        type: 'confirm',
        name: 'install',
        message: 'Install ' + name + ' now?',
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          inquirer.prompt([{
            type: 'list',
            choices: ['Install Globally', 'Install Locally'],
            name: 'global',
            message: 'Install ' + name + ' globally or locally?',
            default: 'Globally'
          }]).then(function (ans) {
            if (ans.global === 'Install Globally') {
              console.log(chalk.blue('Installing ' + name + ' globally via npm...'));
              exec$1('npm i -g ' + module, function (error) {
                console.log(chalk.green('Added ' + name + ' globally.'));
                if (error !== null) {
                  console.log('exec error: ' + error);
                }
                cb();
              });
            } else if (ans.global === 'Install Locally') {
              console.log(chalk.blue('Installing ' + name + ' locally via npm...'));
              exec$1('npm i -D ' + module, function (error) {
                console.log(chalk.green('Added ' + name + ' locally.'));
                if (error !== null) {
                  console.log('exec error: ' + error);
                }
                cb();
              });
            }
          });
        } else {
          console.log(chalk.yellow(name + ' was not installed.'));
        }
      });
    } else {
      console.log(chalk.green(name + ' already exists.'));
      cb();
    }
  });
})

// ----------------------------------------------------------------
var addTape = (function (callback) {
  installModule('tape', 'Tape', function () {
    callback();
  });
})

var exec = childProcess.exec;
// ----------------------------------------------------------------
var runTests = (function (dir) {
  addTape(function () {
    fs.stat(path.resolve('./node_modules/babel-preset-es2015'), function (err) {
      if (err && err.code !== 'ENOENT') console.log(err);
      if (err && err.code === 'ENOENT') {
        var n1 = 'Babel is not installed locally.';
        var n2 = '(To use ES6 with your tests, please install Babel with the Babel preset task.)';
        console.log(chalk.yellow(n1 + '\n' + n2));
        console.log(chalk.yellow('----------------'));
        console.log(chalk.yellow('Running project tests with ES5...'));
      } else {
        console.log(chalk.yellow('Running project tests with ES6...'));
      }
    });
    var nyc = path.resolve(__dirname, '../node_modules/.bin/nyc');
    var bReg = path.resolve(__dirname, '../node_modules/babel-register');
    var tapeL = path.resolve(__dirname, '../node_modules/.bin/tape');
    var dirP = path.resolve(dir);
    var faucet = path.resolve(__dirname, '../node_modules/.bin/faucet');
    var report = '--reporter=lcov --reporter=text';
    var cmdString = nyc + ' --require ' + bReg + ' ' + report + ' ' + tapeL + ' \'' + dirP + '/**/*.js\' | ' + faucet;
    var cmd = exec(cmdString, function (error) {
      if (error !== null) {
        console.log('exec error: ' + error);
      }
    });
    cmd.stdout.pipe(process.stdout);
    cmd.stderr.pipe(process.stderr);
  });
})

var exec$2 = childProcess.exec;
// ----------------------------------------------------------------
var rollupFiles = (function (file, output) {
  console.log('Rolling files into bundle...');
  var env = 'BABEL_ENV=build';
  var rp = path.resolve(__dirname, '../node_modules/.bin/rollup');
  var rpConfig = path.resolve(__dirname, '../vendor/rollup.config.js');
  var fx = path.resolve(file);
  var op = path.resolve(output);
  var cmdString = env + ' ' + rp + ' --config ' + rpConfig + ' ' + fx + ' -o ' + op;
  exec$2(cmdString, function (error) {
    if (error !== null) {
      console.log('exec error: ' + error);
    }
    console.log('Rolled ' + file + ' into bundle at ' + output + '.');
  });
})

// ----------------------------------------------------------------
var addBabelPreset = (function () {
  var addBabelRc = function addBabelRc() {
    fs.stat(path.resolve('.babelrc'), function (err) {
      if (err && err.code !== 'ENOENT') console.log(err);
      if (err && err.code === 'ENOENT') {
        console.log(chalk.yellow('No .babelrc file detected...'));
        console.log(chalk.blue('Creating .babelrc file...'));
        var oldFile = fs.createReadStream(path.resolve(__dirname, '../vendor/.babelrc'));
        var newFile = fs.createWriteStream(path.resolve('.babelrc'));
        oldFile.pipe(newFile);
        console.log(chalk.green('.babelrc file created.'));
      } else {
        console.log(chalk.green('.babelrc already exists.'));
      }
    });
  };
  installModule('babel-preset-es2015', '-D', 'Babel Preset', function () {
    installModule('babel-preset-es2015-rollup', '-D', 'Babel Preset for Rollup', function () {
      addBabelRc();
    });
  });
})

// ----------------------------------------------------------------
var addWallabyConfig = (function () {
  console.log(__dirname);
  fs.stat(path.resolve('wallaby.js'), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      console.log(chalk.yellow('No wallaby.js file detected...'));
      console.log(chalk.blue('Creating wallaby.js file...'));
      var oldFile = fs.createReadStream(path.resolve(__dirname, '../vendor/wallaby.js'));
      var newFile = fs.createWriteStream(path.resolve('wallaby.js'));
      oldFile.pipe(newFile);
      console.log(chalk.green('wallaby.js file created.'));
      console.log(chalk.yellow('Note: Wallaby.js requires babel-core to run ES6 files.'));
    } else {
      console.log(chalk.green('wallaby.js already exists.'));
    }
  });
})

// ----------------------------------------------------------------
var addEzyConfig = (function (callback) {
  fs.stat(path.resolve('ezy.config.json'), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      console.log(chalk.yellow('No ezy.config.json file detected...'));
      console.log(chalk.blue('Creating ezy.config.json file...'));
      var oldFile = fs.createReadStream(path.resolve(__dirname, '../vendor/ezy.config.json'));
      var newFile = fs.createWriteStream(path.resolve('ezy.config.json'));
      oldFile.pipe(newFile);
      oldFile.on('end', function () {
        console.log(chalk.green('ezy.config.json file created.'));
        callback();
      });
    } else {
      console.log(chalk.green('ezy.config.json already exists.'));
      callback();
    }
  });
})

// ----------------------------------------------------------------
var addESLint = (function (callback) {
  var modules = 'eslint-config-airbnb eslint-plugin-import eslint-plugin-react eslint-plugin-jsx-a11y eslint';
  installModule(modules, 'ESLint', function () {
    callback();
  });
})

// ----------------------------------------------------------------
var addESLintRC = (function () {
  fs.stat(path.resolve('.eslintrc'), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      console.log(chalk.yellow('No .eslintrc file detected...'));
      console.log(chalk.blue('Creating .eslintrc file...'));
      var oldFile = fs.createReadStream(path.resolve(__dirname, '../vendor/.eslintrc'));
      var newFile = fs.createWriteStream(path.resolve('.eslintrc'));
      oldFile.pipe(newFile);
      console.log(chalk.green('.eslintrc file created.'));
    } else {
      console.log(chalk.green('.eslintrc already exists.'));
    }
  });
})

var exec$3 = childProcess.exec;
// ----------------------------------------------------------------
var viewCoverage = (function () {
  console.log(chalk.yellow('Opening coverage report in the browser...'));
  exec$3('open ' + path.resolve('coverage/lcov-report/index.html'), function (error) {
    if (error !== null) {
      console.log('exec error: ' + error);
    }
  });
})

// import childProcess from 'child_process';
// const exec = childProcess.exec;
// ----------------------------------------------------------------
var addBrowserAssets = (function (dir, file, callback) {
  fs.stat(path.resolve(dir + '/index.html'), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      var notify = 'index.html not detected at ' + path.normalize(dir + '/index.html') + '...';
      console.log(chalk.yellow(notify));
      inquirer.prompt([{
        type: 'confirm',
        name: 'install',
        message: 'Create index.html now?',
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          (function () {
            console.log(chalk.blue('Creating index.html...'));
            var oldFile = fs.createReadStream(path.resolve(__dirname, '../vendor/index.html'));
            var newFile = fs.createWriteStream(path.resolve(dir + '/index.html'));

            newFile.on('pipe', function (src) {
              console.error('Writing now...');
              if (src === oldFile) {
                fs.readFile(path.resolve(dir + '/index.html'), function (error, data) {
                  if (error) console.log(error);
                  var ndata = data.toString().split('\n');
                  ndata.splice(5, 1, '  <script src="./' + file + '"></script>');
                  var text = ndata.join('\n');
                  fs.writeFile(path.resolve(dir + '/index.html'), text, function (error1) {
                    if (error1) console.log(error1);
                    console.log(chalk.green('index.html file created.'));
                    callback();
                  });
                });
              }
            });
            oldFile.pipe(newFile);
          })();
        } else {
          console.log(chalk.red('An index.html file is required to run projects in the browser.'));
        }
      });
    } else {
      console.log(chalk.green('index.html exists.'));
      callback();
    }
  });
})

var exec$4 = childProcess.exec;
// ----------------------------------------------------------------
var runProject = {
  node: function node(file) {
    console.log(chalk.yellow('Running ' + file + ' with Node...'));
    var cmd = exec$4('node ' + path.resolve(file), function (error) {
      if (error !== null) {
        console.log('exec error: ' + error);
      }
    });
    cmd.stdout.pipe(process.stdout);
    cmd.stderr.pipe(process.stderr);
  },
  es6: function es6(file) {
    console.log(chalk.yellow('Running ' + file + ' with ES6...'));
    var env = 'BABEL_ENV=build';
    var rp = path.resolve(__dirname, '../node_modules/.bin/rollup');
    var rpConfig = path.resolve(__dirname, '../vendor/rollup.config.js');
    var fx = path.resolve(file);
    var cmdString = env + ' ' + rp + ' --config ' + rpConfig + ' ' + fx + ' | node';
    var cmd = exec$4(cmdString, function (error) {
      if (error !== null) {
        console.log('exec error: ' + error);
      }
      // stderr.pipe(process.stderr);
    });
    cmd.stdout.pipe(process.stdout);
    cmd.stderr.pipe(process.stderr);
  },
  browser: function browser(dir, file) {
    addBrowserAssets(dir, file, function () {
      console.log(chalk.yellow('Running ' + dir + ' with BrowserSync...'));
      var bsPath = path.resolve(__dirname, '../node_modules/.bin/browser-sync');
      var dirPath = path.resolve(dir);
      var cmdString = bsPath + ' start --server ' + dirPath + ' --files ' + dirPath;
      var cmd = exec$4(cmdString, function (error) {
        if (error !== null) {
          console.log('exec error: ' + error);
        }
      });
      cmd.stdout.pipe(process.stdout);
      cmd.stderr.pipe(process.stderr);
    });
  }
};

// ----------------------------------------------------------------
var updateDependencies = {
  check: function check() {
    console.log(chalk.yellow('Checking for new versions of your dependencies...'));
    ncu.run({
      packageFile: path.resolve('package.json'),
      jsonUpgraded: true,
      upgrade: false
    }).then(function (toUpgrade) {
      if (Object.getOwnPropertyNames(toUpgrade).length < 1) {
        console.log(chalk.green('Everything is up to date. :)'));
      } else {
        console.log(toUpgrade);
      }
    });
  },
  run: function run() {
    console.log(chalk.yellow('Updating your dependencies...'));
    ncu.run({
      packageFile: path.resolve('package.json'),
      jsonUpgraded: true,
      upgrade: true
    }).then(function (toUpgrade) {
      if (Object.getOwnPropertyNames(toUpgrade).length < 1) {
        console.log(chalk.green('Everything is up to date. :)'));
      } else {
        console.log(toUpgrade);
      }
    });
  }

};

// ----------------------------------------------------------------

var questions = [{
  type: 'list',
  name: 'inputValue',
  message: 'What would you like to do?',
  choices: ['1. Run Project', new inquirer.Separator(), '2. Test Project', '3. View Coverage', new inquirer.Separator(), '4. Bundle Project', new inquirer.Separator(), '5. Check Updates', '6. Run Updates', '7. Add Babel Preset', '8. Add Wallaby Config', '9. Add ESLint', '10. Add ezy Config', new inquirer.Separator()]
}];

var taskList = (function (config) {
  // --------------------------------
  inquirer.prompt(questions).then(function (answers) {
    var a = answers.inputValue;

    if (a === '1. Run Project') {
      inquirer.prompt([{
        type: 'list',
        name: 'runWhere',
        message: 'Where would you like to run the project?',
        choices: ['ES6', 'Node', 'Browser']
      }]).then(function (ans) {
        if (ans.runWhere === 'Node') {
          runProject.node(config.outputDir + '/' + config.output);
        } else if (ans.runWhere === 'Browser') {
          runProject.browser(config.outputDir, config.output);
        } else if (ans.runWhere === 'ES6') {
          runProject.es6(config.entryDir + '/' + config.entry);
        }
      });
    } else if (a === '2. Test Project') {
      runTests(config.testDir, config.entryDir);
    } else if (a === '3. View Coverage') {
      viewCoverage();
    } else if (a === '4. Bundle Project') {
      rollupFiles(config.entryDir + '/' + config.entry, config.outputDir + '/' + config.output);
    } else if (a === '5. Check Updates') {
      updateDependencies.check();
    } else if (a === '6. Run Updates') {
      updateDependencies.run();
    } else if (a === '7. Add Babel Preset') {
      addBabelPreset();
    } else if (a === '8. Add Wallaby Config') {
      addWallabyConfig();
    } else if (a === '9. Add ESLint') {
      addESLint(function () {
        addESLintRC();
      });
    } else if (a === '10. Add ezy Config') {
      addEzyConfig();
    }
  });
})

// ----------------------------------------------------------------
var addEntry = (function (config, callback) {
  // /*eslint-disable*/
  // const config = require(configure); // Weird Require.
  // /*eslint-enable*/
  fs.stat(path.resolve(config.entryDir + '/' + config.entry), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      console.log(chalk.red('No entry file (' + config.entryDir + '/' + config.entry + ') has been found.'));
      inquirer.prompt([{
        type: 'confirm',
        name: 'install',
        message: 'Create your entry file now?',
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          fs.mkdir(path.resolve(config.entryDir), function () {
            var str = 'console.log(\'Hello World\')';
            fs.writeFile(path.resolve(config.entryDir + '/' + config.entry), str, function () {
              console.log(chalk.green('Entry file created.'));
              callback();
            });
          });
        } else {
          console.log(chalk.red('Warning: Skipping entry file creation.'));
          callback();
        }
      });
    } else {
      callback();
    }
  });
})

// ----------------------------------------------------------------
var makeDirectory = (function (name) {
  fs.mkdir(name, function () {
    console.log(chalk.green(name + ' project directory created.'));
    console.log(chalk.green('Now run: \'cd ' + name + '; npm init --yes\''));
  });
})

var addConfigObj = (function (cfg) {
  /* eslint-disable*/
  var config = require(cfg); // Weird Require.
  /* eslint-enable*/
  var entry = void 0;
  var entryDir = void 0;
  var outputDir = void 0;
  var output = void 0;
  var testDir = void 0;
  if (config.entry) {
    entry = config.entry;
  } else {
    entry = 'index.js';
  }
  if (config.entryDir) {
    entryDir = config.entryDir;
  } else {
    entryDir = 'src';
  }
  if (config.outputDir) {
    outputDir = config.outputDir;
  } else {
    outputDir = 'dist';
  }
  if (config.output) {
    output = config.output;
  } else {
    output = 'bundle.js';
  }
  if (config.testDir) {
    testDir = config.testDir;
  } else {
    testDir = 'test';
  }
  return {
    entry: entry,
    entryDir: entryDir,
    output: output,
    outputDir: outputDir,
    testDir: testDir
  };
})

// ----------------------------------------------------------------
var checkForConfig = (function (callback) {
  fs.stat(path.resolve('ezy.config.json'), function (err) {
    if (err && err.code !== 'ENOENT') console.log(err);
    if (err && err.code === 'ENOENT') {
      console.log(chalk.red('No ezy.config.json file has been found.'));
      inquirer.prompt([{
        type: 'confirm',
        name: 'install',
        message: 'Create an ezy.config.json file now?',
        default: true
      }]).then(function (answers) {
        if (answers.install === true) {
          addEzyConfig(function () {
            var config = addConfigObj(path.resolve('ezy.config.json'));
            callback(config);
          });
        } else {
          console.log('Please create a config file to use ezy.');
        }
      });
    } else {
      var config = addConfigObj(path.resolve('ezy.config.json'));
      callback(config);
    }
  });
})

// ----------------------------------------------------------------
var bin = function bin() {
  checkForConfig(function (cfg) {
    addEntry(cfg, function () {
      taskList(cfg);
    });
  });
};
var bundle = function bundle() {
  checkForConfig(function (cfg) {
    addEntry(cfg, function () {
      rollupFiles(cfg.entryDir + '/' + cfg.entry, cfg.outputDir + '/' + cfg.output);
    });
  });
};

var run = {
  node: function node() {
    var x = checkForConfig(function (cfg) {
      addEntry(cfg, function () {
        runProject.node(cfg.outputDir + '/' + cfg.output);
      });
    });
    return x;
  },
  browser: function browser() {
    var x = checkForConfig(function (cfg) {
      addEntry(cfg, function () {
        runProject.browser(cfg.outputDir, cfg.output);
      });
    });
    return x;
  },
  es6: function es6() {
    var x = checkForConfig(function (cfg) {
      addEntry(cfg, function () {
        runProject.es6(cfg.entryDir + '/' + cfg.entry);
      });
    });
    return x;
  }
};

var makeDir = makeDirectory;

exports.bin = bin;
exports.bundle = bundle;
exports.run = run;
exports.makeDir = makeDir;